#!/bin/bash


# check if string only contains [0-9].
# return: 0 if $1 is valid version number.
is_valid_version_number()
{
  echo ${1} | grep -q "^[0-9rc]*$"
}


# parse the version string in the first line of the file $1
# the version numberes are returned in the global variables
# ver1, ver2, ver3, ver4.
# Format:  <minor>.<major>[.<opt>][-<nr>-<commit>][-<dirty>]
# return: 1 on success, 0 on error.
getversion()
{
  # tokenize first line of input file (1, 2, 3 or 4 tokens allowed)
  IFS="-"
  read v1 v2 v3 v4 v5 <<- EOF
	$(head -n 1 "$1")
	EOF

  if [ "$v1" != "" ] && [ "$v2" = "" ]
  then
    # format <major>.<minor>[.<opt>]
    ver4=0
  elif [ "$v2" != "" ] && [ "$v3" = "" ]
  then
    # format <minor>.<major>[.<opt>]-<dirty>
    if [ "$v2" != "dirty" ]
    then
      return 0
    fi
    ver4=0
  elif [ "$v3" != "" ] && [ "$v4" = "" ]
  then
    # format <minor>.<major>[.<opt>]-<nr>-<commit>
    if ! is_valid_version_number "$v2"
    then
      return 0
    fi
    ver4=$v2
  elif [ "$v4" != "" ] && [ "$v5" = "" ]
  then
    # format <minor>.<major>[.<opt>]-<nr>-<commit>-<dirty>
    if ! is_valid_version_number "$v2"
    then
      # v2 is not a number
      return 0
    fi
    if [ "$v4" != "dirty" ]
    then
      return 0
    fi
    ver4=$v2
  else
    # invalid format
    return 0
  fi

  # tokenize x1.x2[.x3] 
  IFS="."
  read x1 x2 x3 x4 <<- EOF
	$(echo "$v1")
	EOF

  if [ "$x4" != "" ]
  then
    # More than three tokens separated by "."
    return 0
  else
    is_valid_version_number "$x1"
    b1=$?
    is_valid_version_number "$x2"
    b2=$?
    if [ "$x3" != "" ]
    then
      is_valid_version_number "$x3"
      b3=$?
      if test $b1 != 0 -o $b2 != 0 -o $b3 != 0
      then
        # x1, x2, or x3 is not a number
        return 0
      fi
    else
      x3=0
      if test $b1 != 0 -o $b2 != 0
      then
        # x1 or x2 is not a number
        return 0
      fi
    fi
  fi

  ver1=$x1
  ver2=$x2
  ver3=$x3
 
# debugging 
# echo $ver1.$ver2.$ver3.$ver4

  return 1
}

first_tuple_higher_than_second_one()
{
    v11=$1
    v12=$2
    v21=$3
    v22=$4

    if test $v11 -gt $v21
    then
        return 0
    elif test $v11 -lt $v21
    then
        return 1
    elif test $v12 -gt $v22
    then
        return 0
    elif test $v12 -lt $v22
    then
        return 2
    else
        # tuples are equal
        return 3
    fi
}

first_version_higher_than_second_one()
{
    if echo "$1" | grep -q "rc"
    then
        if echo "$2" | grep -q "rc"
        then
            first_tuple_higher_than_second_one \
                ${1/rc*/} ${1/*rc/} \
                ${2/rc*/} ${2/*rc/}
        elif test ${1/rc*/} -eq ${2}
        then
            # if 1st param is rc and number before rc is equal with 2nd param
            # then the non rc version (2nd parameter) is higher
            return 1
        else
            test ${1/rc*/} -gt ${2}
        fi
    else
        if ! echo "$2" | grep -q "rc"
        then
            test $1 -gt $2
        elif test ${1} -eq ${2/rc*/}
        then
            # as comment above, with switched garde
            return 0
        else
            test ${1} -gt ${2/rc*/}
        fi
    fi
    return $?
}

case $1 in

"-d")
  if [ "$2" != "" ] && [ "$3" = "" ]
  then 
    head -n 1 "$2"
    exit 0
  fi
  ;;

"-v")
  if [ "$2" != "" ] && [ "$3" = "" ]
  then
    getversion "$2"
    if [ $? = 1 ]
    then
      exit 0
    else
      exit 1
    fi
  fi
  ;;

"-c")
  if [ "$3" != "" ] && [ "$4" = "" ]
  then
    getversion "$2"
    if [ $? = 0 ]
    then
      exit 254
    fi
    v11=$ver1
    v12=$ver2
    v13=$ver3
    v14=$ver4
    getversion "$3"
     if [ $? = 0 ]
    then
      exit 254
    fi
    v21=$ver1
    v22=$ver2
    v23=$ver3
    v24=$ver4
    if first_version_higher_than_second_one "$v11" "$v21"
    then
      exit 1
    elif first_version_higher_than_second_one "$v21" "$v11"
    then
      exit 255
    else
      if first_version_higher_than_second_one "$v12" "$v22"
      then
        exit 1
      elif first_version_higher_than_second_one "$v22" "$v12"
      then
        exit 255
      else
        if first_version_higher_than_second_one "$v13" "$v23"
        then
          exit 1
        elif first_version_higher_than_second_one "$v23" "$v13"
        then
          exit 255
        else
          if first_version_higher_than_second_one "$v14" "$v24"
          then
            exit 1
          elif first_version_higher_than_second_one "$v24" "$v14"
          then
            exit 255
          else
            exit 0
          fi
        fi
      fi
    fi
  fi
  ;;

*)
  ;;

esac

  echo "Usage:"
  echo "    checkversion -d file"
  echo "        Print version string (first line in file) to stdout"
  echo "        return: 0"
  echo "    checkversion -v file"
  echo "        Verify version string (first line in file)"
  echo "        return: 0 for valid version string, else 1"
  echo "    checkversion -c file1 file2"
  echo "        Compare two version strings (first line in each file)"
  echo "        return: 255 if file2 contains most recent version, 1 if file1"
  echo "        contains most recent version, 0 if versions are equivalent,"
  echo "        254 if a parsing error occured"
  exit 1

